#include <Servo.h>
#include <PS2X_lib.h>
#include <EEPROM.h>

//PS2 input pins
#define PS2_DAT 8
#define PS2_CMD 7
#define PS2_ATT 6
#define PS2_CLK 9

//ps2 analog joystick Deadzone
#define Deadzone 10

//Mandible pins
#define Neck_pin 2
#define Head_pin 3
#define Jawr_pin 4
#define Jawl_pin 5

//Pan And Tilt pins
#define Pan_pin 10
#define Tilt_pin 12

//Onboard Speaker
#define Speaker_pin 13

//Pan and Tilt Speed Variable
float PTSpeed = 1.0;

// Servo Objects
Servo Mandturn;
Servo Mandud;
Servo Mandrmand;
Servo Mandlmand;
Servo Pan;
Servo Tilt;

int PlyStnPadRt = 0;
int PlyStnPadLt = 0;
int PlyStnPadUp = 0;
int PlyStnPadDown = 0;


int PlyStnRStickUpDn = 0; //Value read off the PS2 Right Stick up/down
int PlyStnRStickLtRt = 0; //Value read off the PS2 Right Stick left/right
int PlyStnLStickUpDn = 0; //Value read off the PS2 Left Stick up/down
int PlyStnLStickLtRt = 0; //Value read off the PS2 Left Stick Left/right

int MandturnSetting = 90; //Setting for Mandible turn servo
int MandudSetting = 90; //Setting for Mandible up/down servp
int MandrmandSetting = 90; //Setting for right Mandible servo
int MandlmandSetting = 90; //Setting for left Mandible servo

// Pan And Tilt variables
int P = 0;
int T = 0;
#define PAN_MAX    170     //Maximum angle allowed (pan)
#define PAN_MIN    10      //Minimum angle allowed (pan)
#define TILT_MAX   170     //Maximum angle allowed (tilt)
#define TILT_MIN   70      //Minimum angle allowed (tilt)
#define PAN_STEP   1       //Amount of degrees per move (first press) (pan)
#define PAN_CONT   1       //Amount of degress per move (held) (pan)
#define TILT_STEP  1       //Amount of degress per move (first press) (tilt)
#define TILT_CONT  1       //Amount of degress per move (held) (tilt)
bool P_heldInc = false;
bool P_heldDec = false;
bool T_heldInc = false;
bool T_heldDec = false;

// PS2X Variables
PS2X ps2x;
int error = 0;
byte type = 0;

//Offsets
byte Offsets[2] = {127, 127};

uint32_t PS2_ErrorCount = 0;

boolean mode = true;

void setup() {
  ps2x.config_gamepad(9,7,6,8, false, false);
  
  //Attach Mandible pins
  Mandturn.attach(Neck_pin);
  Mandud.attach(Head_pin);
  Mandrmand.attach(Jawr_pin);
  Mandlmand.attach(Jawl_pin);

  //Attach Pan And Tilt pins
  Pan.attach(Pan_pin);
  Tilt.attach(Tilt_pin);
 
//Set Mandible servos to 90 degrees
  Mandturn.write(90);
  Mandud.write(90);
  Mandrmand.write(90);
  Mandlmand.write(90);

  //Set Pan and Tilt servos to 90 degrees
  Pan.write(90);
  Tilt.write(90);
  delay(2000);
}

void loop() {
  //Update the ps2 controller
  ps2x.read_gamepad();

  //Check if gamepad read was valid
  if ((ps2x.Analog(1) & 0xf0) == 0x70)
  {
    //Reset error count
    PS2_ErrorCount = 0;
  }

  PlyStnPadRt = ps2x.ButtonPressed(PSB_PAD_RIGHT);
  PlyStnPadLt = ps2x.ButtonPressed(PSB_PAD_LEFT);
  PlyStnPadUp = ps2x.ButtonPressed(PSB_PAD_UP);
  PlyStnPadDown = ps2x.ButtonPressed(PSB_PAD_DOWN);
 
  //Analog Stick readings
  PlyStnRStickUpDn = 128 - ps2x.Analog(PSS_RY);
  PlyStnRStickLtRt = ps2x.Analog(PSS_RX) - 128;
  PlyStnLStickUpDn = 128 - ps2x.Analog(PSS_LY);
  PlyStnLStickLtRt = ps2x.Analog(PSS_LX) - 128;

  //Readings from ps2 Controller Sticks are from 0 to 255 with the neutral being 128. The zero positions are to the left for x-axis movements and up for y-axis movements.
  //Variables to carry the settings for the servos. The values from the ps2 sticks are mapped to 0 to 180 degrees.

  MandturnSetting = map(PlyStnRStickUpDn,-256,256,0,160);
  MandudSetting = map(PlyStnRStickLtRt,-256,256,0,160);
  MandrmandSetting = map(PlyStnLStickUpDn,-256,256,0,160);
  MandlmandSetting = map(PlyStnLStickUpDn,-256,256,160,0);

//Write it to servos

Mandturn.write(MandturnSetting);
Mandud.write(MandudSetting);
Mandrmand.write(MandrmandSetting);
Mandlmand.write(MandlmandSetting);

// Pan And Tilt Control
  if(ps2x.ButtonPressed(PSB_PAD_RIGHT))
{
  P = min(P + PAN_STEP*PTSpeed, PAN_MAX);
  Pan.write(P);

  //indicate the button for incrementing pan is pressed
  P_heldInc = true;
}
else if(ps2x.ButtonPressed(PSB_PAD_LEFT))
{
  P = max(P - PAN_STEP*PTSpeed, PAN_MIN);
  Pan.write(P);

  //indicate the button for decrementing pan is pressed
  P_heldDec = true;
}
  if(ps2x.ButtonPressed(PSB_PAD_UP))
{
  T = min(T + TILT_STEP*PTSpeed, TILT_MAX);
  Tilt.write(T);

  //indicate the button for incrementing tilt is pressed
  T_heldInc = true;
}
else if(ps2x.ButtonPressed(PSB_PAD_DOWN))
{
  T = max(T - TILT_STEP*PTSpeed, TILT_MIN);
  Tilt.write(T);

  //indicate the button for decrementing tilt is pressed
  T_heldDec = true;
}
//if they are not pressed, check if it has been released
else
{
  //check pan incrementing
    if(ps2x.ButtonReleased(PSB_PAD_RIGHT))
  {
    P_heldInc = false;
  }
  //Check pan decrementing
    if(ps2x.ButtonReleased(PSB_PAD_LEFT))
  {
    P_heldDec = false;
  }
  //Check tilt incrementing
    if(ps2x.ButtonReleased(PSB_PAD_UP))
{
  T_heldInc = false;
}
//Check tilt decrementing
  if(ps2x.ButtonPressed(PSB_PAD_DOWN))
{
  T_heldDec = false;
}

//Modify values accordingly
  if(P_heldInc)
{
  P = min(P + PAN_CONT*PTSpeed, PAN_MAX);
    Pan.write(P);
  }
  if(P_heldDec)
{
  P = max(P - PAN_CONT*PTSpeed, PAN_MIN);
    Pan.write(P);
  }
  if(T_heldInc)
{
  T = min(T + TILT_CONT*PTSpeed, TILT_MAX);
    Tilt.write(T);
  }
  if(T_heldDec)
{
  T = max(T - TILT_CONT*PTSpeed, TILT_MIN);
    Tilt.write(T);
  }
}

}
